package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.CrossInOutEnum;
import net.wanji.common.enums.StrategyAndMetricsEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.databus.vo.CrossIdAndNameVO;
import net.wanji.opt.bo.*;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.dao.mapper.trend.GreenwaveInfoMapper;
import net.wanji.opt.po.strategy.SceneStrategyPO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.service.MainlineEvaluateService;
import net.wanji.opt.vo.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.ToDoubleFunction;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/9/19 15:24
 */
@Slf4j
@Service
public class MainlineEvaluateServiceImpl implements MainlineEvaluateService {
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final BaseCrossTurnInfoMapper baseCrossTurnInfoMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossDataHistMapper crossDataHistMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final GreenwaveInfoMapper greenwaveInfoMapper;
    private final GreenwaveHistMapper greenwaveHistMapper;
    private final GreenwaveCrossMapper greenwaveCrossMapper;
    private final BaseCrossInfoMapper baseCrossInfoMapper;
    private final GreenwaveSceneMapper greenwaveSceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final StrategyMapper strategyMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final RidInfoMapper ridInfoMapper;

    SimpleDateFormat hourMinuteFormat = new SimpleDateFormat("HH:mm");
    SimpleDateFormat dayFormat = new SimpleDateFormat("yyyy-MM-dd");
    SimpleDateFormat dateHourMinuteFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public MainlineEvaluateServiceImpl(@Qualifier("baseCrossDirInfoMapper") BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                                       @Qualifier("baseCrossTurnInfoMapper") BaseCrossTurnInfoMapper baseCrossTurnInfoMapper,
                                       CrossBaseLaneInfoMapper crossBaseLaneInfoMapper,
                                       @Qualifier("laneInfoMapper") LaneInfoMapper laneInfoMapper,
                                       CrossDirDataHistMapper crossDirDataHistMapper, CrossDataHistMapper crossDataHistMapper, CrossTurnDataHistMapper crossTurnDataHistMapper, CrossLaneDataHistMapper crossLaneDataHistMapper, @Qualifier("greenwaveInfoMapper") GreenwaveInfoMapper greenwaveInfoMapper, @Qualifier("greenwaveHistMapper") GreenwaveHistMapper greenwaveHistMapper, @Qualifier("greenwaveCrossMapper") GreenwaveCrossMapper greenwaveCrossMapper, @Qualifier("baseCrossInfoMapper") BaseCrossInfoMapper baseCrossInfoMapper, @Qualifier("greenwaveSceneMapper") GreenwaveSceneMapper greenwaveSceneMapper, @Qualifier("sceneStrategyMapper") SceneStrategyMapper sceneStrategyMapper, @Qualifier("strategyMapper") StrategyMapper strategyMapper, @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper, @Qualifier("ridInfoMapper") RidInfoMapper ridInfoMapper) {
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.baseCrossTurnInfoMapper = baseCrossTurnInfoMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.crossDataHistMapper = crossDataHistMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
        this.crossLaneDataHistMapper = crossLaneDataHistMapper;
        this.greenwaveInfoMapper = greenwaveInfoMapper;
        this.greenwaveHistMapper = greenwaveHistMapper;
        this.greenwaveCrossMapper = greenwaveCrossMapper;
        this.baseCrossInfoMapper = baseCrossInfoMapper;
        this.greenwaveSceneMapper = greenwaveSceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.strategyMapper = strategyMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.ridInfoMapper = ridInfoMapper;
    }

    @Override
    public List<String> bottomMenu(BottomMenuBO bo) {
        String crossId = bo.getCrossId();
        Integer scope = bo.getScope();

        if (scope == 1) {
            return buildDirMenu(crossId);
        } else if (scope == 2) {
            return buildTurnMenu(crossId);
        } else if (scope == 3) {
            return buildLaneMenu(crossId);
        }

        return null;
    }

    @Override
    public List<MainlineEvaluateBottomCurveVO> bottomCurve(BottomCurveBO bo) throws ParseException {
        String crossId = bo.getCrossId();
        Integer scope = bo.getScope();
        List<String> scopeList = bo.getScopeList();
        Integer minutes = bo.getMinutes();
        String metricCode = bo.getMetricCode();
        Date startTime = bo.getStartTime();
        Date endTime = bo.getEndTime();

        if (scope != 1 && Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode())) {
            throw new RuntimeException("只有路口级别可选择负载均衡度");
        }

        List<String> timeList = buildTimeList(startTime, endTime, minutes);
        List<MainlineEvaluateBottomCurveVO> res = new ArrayList<>();


        int startTimeStamp = (int) (startTime.getTime() / 1000);
        int endTimeStamp = (int) (endTime.getTime() / 1000);

        if (scope == 0) { // 路口
            // 查询路口该时段内的所有数据
            List<CrossDataHistPO> crossPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                    crossId, startTimeStamp, endTimeStamp);
            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);
                MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                vo.setMetricTime(timeStr);

                // 计算 endDatetime
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(startDatetime);
                calendar.add(Calendar.MINUTE, minutes);
                Date endDatetime = calendar.getTime();
                // 转换为 10 位时间戳
                int startTimeStampPart = (int) (startDatetime.getTime() / 1000);
                int endTimeStampPart = (int) (endDatetime.getTime() / 1000);
                // 使用流式编程进行筛选
                List<CrossDataHistPO> filteredList = crossPOList.stream()
                        .filter(po -> po.getBatchTime() >= startTimeStampPart && po.getBatchTime() <= endTimeStampPart)
                        .collect(Collectors.toList());
                if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getNoStopRate)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getOneStopRate)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                    double v = filteredList.stream()
                            .mapToInt(CrossDataHistPO::getDelayTime)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getQueueLength)
                            .max()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getStopTimes)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getSpeed)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getGreenLightEfficiency)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getSturation)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v * 100));
                    vo.setValue(round);
                } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode())) {
                    double v = filteredList.stream()
                            .mapToDouble(CrossDataHistPO::getLoadBalance)
                            .average()
                            .orElse(0.0);
                    int round = (int) (Math.round(v));
                    vo.setValue(round);
                }

                res.add(vo);
            }
        } else if (scope == 1) { // 进口道
            // 查询各进口道该时段内的所有数据
            List<CrossDirDataHistPO> dirPOList = crossDirDataHistMapper.selectByCrossIdAndStartEnd(
                    crossId, startTimeStamp, endTimeStamp);

            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);

                for (String dirStr : scopeList) {
                    int index = dirStr.indexOf("进口");
                    String dirName = dirStr.substring(0, index);
                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(dirStr);

                    // 计算 endDatetime
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(startDatetime);
                    calendar.add(Calendar.MINUTE, minutes);
                    Date endDatetime = calendar.getTime();
                    // 转换为 10 位时间戳
                    int startTimeStampPart = (int) (startDatetime.getTime() / 1000);
                    int endTimeStampPart = (int) (endDatetime.getTime() / 1000);
                    // 使用流式编程进行筛选
                    List<CrossDirDataHistPO> filteredList = dirPOList.stream()
                            .filter(po -> po.getDirType().equals(dirCode))
                            .filter(po -> po.getInOutType().equals(1))
                            .filter(po -> po.getBatchTime() >= startTimeStampPart
                                    && po.getBatchTime() <= endTimeStampPart)
                            .collect(Collectors.toList());
                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossDirDataHistPO::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossDirDataHistPO::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        } else if (scope == 2) { // 方向
            // 查询各方向该时段内的所有数据
            List<CrossTurnDataHistPO> dirPOList = crossTurnDataHistMapper.selectByCrossId(
                    crossId, endTimeStamp, startTimeStamp);

            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);

                for (String turnStr : scopeList) {
                    int index = turnStr.indexOf("进口");
                    String dirName = turnStr.substring(0, index);
                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);
                    String turnName = turnStr.substring(index + 2);
                    String turnCode = TurnConvertEnum.getCodeByDesc(turnName);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(turnStr);

                    // 计算 endDatetime
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(startDatetime);
                    calendar.add(Calendar.MINUTE, minutes);
                    Date endDatetime = calendar.getTime();
                    // 转换为 10 位时间戳
                    int startTimeStampPart = (int) (startDatetime.getTime() / 1000);
                    int endTimeStampPart = (int) (endDatetime.getTime() / 1000);
                    // 使用流式编程进行筛选
                    List<CrossTurnDataHistPO> filteredList = dirPOList.stream()
                            .filter(po -> po.getInDir().equals(dirCode))
                            .filter(po -> po.getTurnType().equals(turnCode))
                            .filter(po -> po.getBatchTime() >= startTimeStampPart
                                    && po.getBatchTime() <= endTimeStampPart)
                            .collect(Collectors.toList());
                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossTurnDataHistPO::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossTurnDataHistPO::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        } else if (scope == 3) { // 车道
            // 查询各车道该时段内的所有数据
            List<CrossLaneDataHistPOExt> lanePOList = crossLaneDataHistMapper.selectByCrossId(
                    crossId, endTimeStamp, startTimeStamp);

            for (String timeStr : timeList) {
                Date startDatetime = calcDate(timeStr, startTime);

                for (String laneStr : scopeList) {
                    int indexOfJinKou = laneStr.indexOf("进口");
                    int indexOfCheDao = laneStr.indexOf("车道");
                    // 截取"进口"之前的部分
                    String dirName = laneStr.substring(0, indexOfJinKou);
                    // 截取"车道"之前的数字部分
                    String laneSortStr = laneStr.substring(indexOfCheDao - 2, indexOfCheDao);

                    Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName);
                    Integer laneSort = Integer.parseInt(laneSortStr);

                    MainlineEvaluateBottomCurveVO vo = new MainlineEvaluateBottomCurveVO();
                    vo.setMetricTime(timeStr);
                    vo.setScopeName(laneStr);

                    // 计算 endDatetime
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(startDatetime);
                    calendar.add(Calendar.MINUTE, minutes);
                    Date endDatetime = calendar.getTime();
                    // 转换为 10 位时间戳
                    int startTimeStampPart = (int) (startDatetime.getTime() / 1000);
                    int endTimeStampPart = (int) (endDatetime.getTime() / 1000);
                    // 筛选方向、车道序号
                    List<CrossLaneDataHistPOExt> filteredList = lanePOList.stream()
                            .filter(po -> po.getDir().equals(dirCode))
                            .filter(po -> po.getSort().equals(laneSort))
                            .filter(po -> po.getBatchTime() >= startTimeStampPart
                                    && po.getBatchTime() <= endTimeStampPart)
                            .collect(Collectors.toList());

                    if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getNoStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getOneStopRate)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode())) {
                        double v = filteredList.stream()
                                .mapToInt(CrossLaneDataHistPOExt::getDelayTime)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getQueueLength)
                                .max()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getStopTimes)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getSpeed)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getGreenLightEfficiency)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    } else if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.SATURATION.getCode())) {
                        double v = filteredList.stream()
                                .mapToDouble(CrossLaneDataHistPOExt::getSturation)
                                .average()
                                .orElse(0.0);
                        int round = (int) (Math.round(v * 100));
                        vo.setValue(round);
                    }

                    res.add(vo);
                }
            }
        }


        return res;
    }

    @Override
    public List<MainlineListVO> mainlineList() {
        List<MainlineListVO> res = new ArrayList<>();

        // 按干线名称过滤数据
        List<GreenwaveInfoPO> poList = greenwaveInfoMapper.selectAll();
        List<GreenwaveInfoPO> filteredList = poList.stream()
                .collect(Collectors.groupingBy(po -> getPrefix(po.getName())))
                .values()
                .stream()
                .filter(list -> !list.isEmpty())
                .map(list -> list.get(0))
                .collect(Collectors.toList());

        for (GreenwaveInfoPO greenwaveInfoPO : filteredList) {
            MainlineListVO mainlineListVO = new MainlineListVO();

            String name = greenwaveInfoPO.getName(); // 例：旅游路-转山西路至霞景路路段:双向绿波1
            String prefix = getPrefix(name);
            mainlineListVO.setMainlineName(prefix);

            Integer greenwaveId = greenwaveInfoPO.getId();
            setCrossListAndwkt(mainlineListVO, greenwaveId);
            res.add(mainlineListVO);
        }

        return res;
    }

    private void setCrossListAndwkt(MainlineListVO mainlineListVO, Integer greenwaveId) {
        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        List<CrossIdAndNameVO> crossIdAndNameVOList = new ArrayList<>();
        StringBuilder wkt = new StringBuilder();

        for (int i = 0; i < greenwaveCrossPOList.size() - 1; i++) {
            GreenwaveCrossPO preCross = greenwaveCrossPOList.get(i);
            GreenwaveCrossPO nextCross = greenwaveCrossPOList.get(i + 1);

            // 构造路口列表
            String crossId = preCross.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
            crossIdAndNameVO.setCrossId(crossId);
            crossIdAndNameVO.setCrossName(baseCrossInfoPO.getName());
            crossIdAndNameVOList.add(crossIdAndNameVO);
            if (i == greenwaveCrossPOList.size() - 2) {
                String crossId1 = nextCross.getCrossId();
                BaseCrossInfoPO baseCrossInfoPO1 = baseCrossInfoMapper.selectById(crossId1);
                CrossIdAndNameVO crossIdAndNameVO1 = new CrossIdAndNameVO();
                crossIdAndNameVO1.setCrossId(crossId1);
                crossIdAndNameVO1.setCrossName(baseCrossInfoPO1.getName());
                crossIdAndNameVOList.add(crossIdAndNameVO1);
            }

            // 构造wkt
            RidInfoEntity ridInfo = ridInfoMapper.selectByStartEnd(preCross.getCrossId(), nextCross.getCrossId());
            if (i == 0) {
                wkt.append(ridInfo.getWkt());
            } else {
                wkt.append(";").append(ridInfo.getWkt());
            }
        }

        mainlineListVO.setCrossList(crossIdAndNameVOList);
        mainlineListVO.setWkt(wkt.toString());
    }


    private String getPrefix(String name) {
        int index = name.indexOf(':');
        return index != -1 ? name.substring(0, index).trim() : name;
    }

    @Override
    public MainlineSchemeAnalysisVO mainlineSchemeAnalysis(MainlineSchemeAnalysisBO bo) {
        String mainlineName = bo.getName();
        Date poStartTime = bo.getStartTime();
        Date poEndTime = bo.getEndTime();
        String poStartTimeStr = sdf.format(poStartTime);
        String poEndTimeStr = sdf.format(poEndTime);
        // 根据时段查询所有发生的绿波
        List<GreenwaveHistPO> greenwaveHistPOList = greenwaveHistMapper
                .selectByTimeSection(poStartTimeStr, poEndTimeStr);
        // 根据干线名称筛选结果
        List<GreenwaveHistPO> filteredList = greenwaveHistPOList.stream()
                .filter(po -> {
                    String name = po.getGreenwaveName();
                    int index = name.indexOf(':');
                    if (index != -1) {
                        String leftSide = name.substring(0, index).trim();
                        return leftSide.equals(mainlineName);
                    }
                    return false;
                })
                .collect(Collectors.toList());

        MainlineSchemeAnalysisVO res = new MainlineSchemeAnalysisVO();
        res.setSceneData(calcSceneData(filteredList));
        res.setCrossData(calcCrossData(filteredList));
        res.setEvaluateData(calcEvaluateData(filteredList));
        res.setGreenwaveData(buildGreenwaveData(filteredList));

        return res;
    }

    @Override
    public MainlineSchemeEvaluateVO mainlineSchemeEvaluate(MainlineSchemeEvaluateBO bo) {
        Integer greenwaveId = bo.getGreenwaveId();
        String dirName = bo.getDirName();
        String metricName = bo.getMetricName();
        Date boStartTime = bo.getStartTime();
        Date boEndTime = bo.getEndTime();

        MainlineSchemeEvaluateVO res = new MainlineSchemeEvaluateVO();

        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        Integer sectionId = greenwaveInfoPO.getSectionId();
        CrossSectionPO crossSectionPO = baseCrossSectionMapper.selectById(sectionId);
        String startHourMinuteStr = crossSectionPO.getStartTime();
        String endHourMinuteStr = crossSectionPO.getEndTime();
        res.setTimeSection(startHourMinuteStr + "-" + endHourMinuteStr);

        // 确定控制策略
        StrategyPO strategyPO = getStrategyPO(greenwaveId);
        String strategyName = strategyPO.getStrategyName();
        res.setStrategyName(strategyName);

        res.setGreenwaveCrossList(buildGreenwaveCrossList(greenwaveId));

        String strategyCode = strategyPO.getStrategyCode();
        setCurveElementListAndEvaluateList(dirName, metricName, greenwaveId, boStartTime, boEndTime, startHourMinuteStr,
                endHourMinuteStr, res, strategyCode);

        return res;
    }

    @Override
    public List<MainlineCrossEvaluateVO> mainlineCrossEvaluate(MainlineCrossEvaluateBO bo) {
        String greenwaveName = bo.getGreenwaveName();
        String metricName = bo.getMetricName();
        List<String> crossIdList = bo.getCrossIdList();
        Date startDate = bo.getStartTime();
        Date endDate = bo.getEndTime();
        String startHourMinute = bo.getStartHourMinute();
        String endHourMinute = bo.getEndHourMinute();

        List<MainlineCrossEvaluateVO> res = new ArrayList<>();

        for (String crossId : crossIdList) {
            MainlineCrossEvaluateVO mainlineCrossEvaluateVO = new MainlineCrossEvaluateVO();

            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            String crossName = baseCrossInfoPO.getName();
            mainlineCrossEvaluateVO.setCrossName(crossName);

            // 确定协调方向
            String[] split = greenwaveName.split(" ");
            String dirStr = split[1];
            String[] split1 = dirStr.split("向");
            GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectByName(greenwaveName);
            Integer dir = greenwaveInfoPO.getDir();

            List<Integer> coordDirCodeList = new ArrayList<>();
            if (dir == 0) { // 正向
                String dirStr1 = split1[0];
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
                dirCode = convertDirCode(dirStr1, crossId, dirCode);
                coordDirCodeList.add(dirCode);
            } else if (dir == 1) { // 反向
                String dirStr1 = split1[1];
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
                dirCode = convertDirCode(dirStr1, crossId, dirCode);
                coordDirCodeList.add(dirCode);
            } else { // 双向
                String dirStr1 = split1[0];
                Integer dirCode1 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
                dirCode1 = convertDirCode(dirStr1, crossId, dirCode1);
                coordDirCodeList.add(dirCode1);
                String dirStr2 = split1[1];
                Integer dirCode2 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr2);
                dirCode2 = convertDirCode(dirStr2, crossId, dirCode2);
                coordDirCodeList.add(dirCode2);
            }
            mainlineCrossEvaluateVO.setCoordValue(calcCoordValue(
                    coordDirCodeList, crossId, startDate, endDate, startHourMinute, endHourMinute));
            mainlineCrossEvaluateVO.setNonCoordValue(calcNonCoordValue(
                    coordDirCodeList, crossId, startDate, endDate, startHourMinute, endHourMinute));
            mainlineCrossEvaluateVO.setDirElementList(buildDirElementList(
                    crossId, metricName, startDate, endDate, startHourMinute, endHourMinute));
            mainlineCrossEvaluateVO.setCrossEvaluateList(buildCrossEvaluateList(
                    crossId, startDate, endDate, startHourMinute, endHourMinute));
            res.add(mainlineCrossEvaluateVO);
        }
        return res;
    }

    private List<String> buildCrossEvaluateList(
            String crossId, Date startDate, Date endDate, String startHourMinute, String endHourMinute) {

        List<String> res = new ArrayList<>();

        // 获取路口所有方向
        List<BaseCrossDirInfoPO> baseCrossDirInfoPOList =
                baseCrossDirInfoMapper.selectByCrossIdAndInOutType(crossId, CrossInOutEnum.IN.getCode());
        List<Integer> dirCodeList = baseCrossDirInfoPOList.stream()
                .map(BaseCrossDirInfoPO::getDirType)
                .collect(Collectors.toList());

        for (Integer dirCode : dirCodeList) {
            String boStartDayStr = dayFormat.format(startDate);
            String boEndDayStr = dayFormat.format(endDate);
            List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossDirAndTimeSection(
                    crossId, dirCode, boStartDayStr, boEndDayStr, startHourMinute, endHourMinute);
            Integer maxDelayTime = crossDirDataHistPOList.stream()
                    .map(CrossDirDataHistPO::getDelayTime)
                    .max(Integer::compareTo)
                    .orElse(0);
            Double maxQueueLength = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getQueueLength)
                    .max()
                    .orElse(0.0);


            int level = CrossUtil.calcEfficiencyLevel(maxDelayTime, maxQueueLength);
            if (level == 3) {
                res.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口延误较高效率较低");
            } else if (level == 4) {
                res.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口延误高效率低");
            }
        }
        if (CollectionUtil.isEmpty(res)) {
            res.add("该路口运行效率良好");
        }

        return res;
    }

    private List<MainlineCrossEvaluateVO.DirElement> buildDirElementList(
            String crossId, String metricName, Date startDate, Date endDate, String startHourMinute,
            String endHourMinute) {

        // 获取路口所有方向
        List<BaseCrossDirInfoPO> baseCrossDirInfoPOList =
                baseCrossDirInfoMapper.selectByCrossIdAndInOutType(crossId, CrossInOutEnum.IN.getCode());
        List<Integer> dirCodeList = baseCrossDirInfoPOList.stream()
                .map(BaseCrossDirInfoPO::getDirType)
                .collect(Collectors.toList());

        List<MainlineCrossEvaluateVO.DirElement> res = new ArrayList<>();
        for (Integer dirCode : dirCodeList) {
            MainlineCrossEvaluateVO.DirElement dirElement = new MainlineCrossEvaluateVO.DirElement();
            dirElement.setDir(dirCode);

            String boStartDayStr = dayFormat.format(startDate);
            String boEndDayStr = dayFormat.format(endDate);
            List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossDirAndTimeSection(
                    crossId, dirCode, boStartDayStr, boEndDayStr, startHourMinute, endHourMinute);
            dirElement.setValue(calcDirValue(metricName, crossDirDataHistPOList));

            res.add(dirElement);
        }
        return res;
    }

    private Integer calcDirValue(String metricName, List<CrossDirDataHistPO> crossDirDataHistPOList) {
        int dirValue = 0;
        if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getGreenLightEfficiency)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average * 100);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getStopTimes)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getDelayTime)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average);
        } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getDescription())) {
            double average = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                    .average()
                    .orElse(0.0);
            dirValue = (int) Math.round(average);
        }
        return dirValue;
    }

    private Integer calcNonCoordValue(List<Integer> coordDirCodeList, String crossId, Date startDate, Date endDate,
                                      String startHourMinute, String endHourMinute) {
        // 筛选非协调方向
        List<BaseCrossDirInfoPO> baseCrossDirInfoPOList =
                baseCrossDirInfoMapper.selectByCrossIdAndInOutType(crossId, CrossInOutEnum.IN.getCode());
        List<Integer> nonCoordDirCodeList = baseCrossDirInfoPOList.stream()
                .map(BaseCrossDirInfoPO::getDirType)
                .collect(Collectors.toList());
        nonCoordDirCodeList.removeAll(coordDirCodeList);

        // 查询时段内路口非协调方向数据
        String boStartDayStr = dayFormat.format(startDate);
        String boEndDayStr = dayFormat.format(endDate);
        List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossDirsAndTimeSection(
                crossId, nonCoordDirCodeList, boStartDayStr, boEndDayStr, startHourMinute, endHourMinute);

        // 计算非协调方向指标(排队长度)
        int nonCoordValue = (int) (crossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getQueueLength)
                .average()
                .orElse(0.0));

        return nonCoordValue;
    }

    private Integer calcCoordValue(List<Integer> coordDirCodeList, String crossId, Date startDate, Date endDate,
                                   String startHourMinute, String endHourMinute) {

        // 查询时段内路口协调方向数据
        String boStartDayStr = dayFormat.format(startDate);
        String boEndDayStr = dayFormat.format(endDate);
        List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossDirsAndTimeSection(
                crossId, coordDirCodeList, boStartDayStr, boEndDayStr, startHourMinute, endHourMinute);

        // 计算协调方向指标(不停车通过率)
        int coordValue = (int) (crossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                .average()
                .orElse(0.0) * 100);

        return coordValue;
    }

    private void setCurveElementListAndEvaluateList(
            String dirName, String metricName, Integer greenwaveId, Date boStartTime, Date boEndTime,
            String startHourMinuteStr, String endHourMinuteStr, MainlineSchemeEvaluateVO res, String strategyCode) {

        String boStartTimeStr = sdf.format(boStartTime);
        String boEndTimeStr = sdf.format(boEndTime);

        // 获取绿波历史数据
        List<GreenwaveHistPO> greenwaveHistPOList = greenwaveHistMapper
                .selectByIdAndTimeSection(greenwaveId, boStartTimeStr, boEndTimeStr);
        // 过滤时段内的数据
        List<GreenwaveHistPO> filteredGreenwaveList = greenwaveHistPOList.stream()
                .filter(po -> {
                    try {
                        return isTimeInRange(po.getGmtModified(), startHourMinuteStr, endHourMinuteStr);
                    } catch (ParseException e) {
                        throw new RuntimeException(e);
                    }
                })
                .collect(Collectors.toList());
        // 获取绿波指标
        double noparkPassRate = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getNoparkPassRate)
                .average()
                .orElse(0.0);
        double cordReliability = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getCordReliability)
                .average()
                .orElse(0.0);
        double uncoordinatePhaseQueue = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getUncoordinatePhaseQueue)
                .average()
                .orElse(0.0);
        double cordQueueRatio = filteredGreenwaveList.stream()
                .mapToDouble(GreenwaveHistPO::getCordQueueRatio)
                .average()
                .orElse(0.0);

        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        // 获取各路口方向历史数据
        // 提取方向
        String[] split = dirName.split("向");
        String dirStr = split[0];
        Integer dir = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr);
        List<CrossDirDataHistPO> totalCrossDirDataHistPOList = new ArrayList<>();
        for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOList) {
            buildTotalCrossDirHistList(boStartTime, boEndTime, startHourMinuteStr, endHourMinuteStr, dirStr, dir,
                    totalCrossDirDataHistPOList, greenwaveCrossPO);
        }

        // 获取协调方向评价指标
        int coordSpeed = 0;
        int cordDelay = 0;
        if (dirName.contains("非协调")) {
            String dirStr1 = split[1];
            String[] split1 = dirStr1.split("（");
            String s = split1[0];
            Integer dir1 = BaseEnum.SignalDirectionEnum.getCodeByName(s);
            List<CrossDirDataHistPO> totalCrossDirDataHistPOList1 = new ArrayList<>();
            for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOList) {
                buildTotalCrossDirHistList(boStartTime, boEndTime, startHourMinuteStr, endHourMinuteStr, dirStr1, dir1,
                        totalCrossDirDataHistPOList1, greenwaveCrossPO);

                coordSpeed = (int) (totalCrossDirDataHistPOList1.stream()
                        .mapToDouble(CrossDirDataHistPO::getSpeed)
                        .average()
                        .orElse(0.0));
                cordDelay = (int) (totalCrossDirDataHistPOList1.stream()
                        .mapToDouble(CrossDirDataHistPO::getDelayTime)
                        .average()
                        .orElse(0.0));
            }
        } else {
            coordSpeed = (int) (totalCrossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                    .average()
                    .orElse(0.0));
            cordDelay = (int) (totalCrossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getDelayTime)
                    .average()
                    .orElse(0.0));
        }

        // 生成评价结果
        List<String> evaluateList = new ArrayList<>();
        // 获取饱和度
        double saturation = totalCrossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getSturation)
                .average()
                .orElse(0.0);
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        Integer greenwaveDir = greenwaveInfoPO.getDir();
        if (greenwaveDir == 0 || greenwaveDir == 1) { // 单向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                if (saturation < 0.8) {
                    if (noparkPassRate > 0.8 && cordReliability > 0.7) {
                        // 协调不停车通过率需大于80%，并且协调方案可靠性需大于70%
                        evaluateList.add("该方案符合控制策略");
                    } else if (uncoordinatePhaseQueue < 0.5) {
                        // 如果以上不满足，则非协调相位二次排队需小于50%
                        evaluateList.add("该方案符合控制策略");
                    } else {
                        evaluateList.add("该方案不符合控制策略");
                    }
                } else {
                    // 协调方案可靠性需大于70%
                    if (cordReliability > 0.7) {
                        evaluateList.add("该方案符合控制策略");
                    } else {
                        evaluateList.add("该方案不符合控制策略");
                    }
                }
            } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                if (cordQueueRatio < 0.6) {
                    // 协调路段排队空间占比需小于60%
                    evaluateList.add("该方案符合控制策略");
                } else if (uncoordinatePhaseQueue < 1.0) {
                    // 如果以上不满足，则非协调相位二次排队需小于100%
                    evaluateList.add("该方案符合控制策略");
                } else {
                    evaluateList.add("该方案不符合控制策略");
                }
            }
        } else { // 双向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                // 协调不停车通过率需大于50%，并且协调方案可靠性需大于70%
                if (noparkPassRate > 0.5 && cordReliability > 0.7) {
                    evaluateList.add("该方案符合控制策略");
                } else {
                    evaluateList.add("该方案不符合控制策略");
                }
            } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                if (cordQueueRatio < 0.6) {
                    // 协调路段排队空间占比需小于60%
                    evaluateList.add("该方案符合控制策略");
                } else if (uncoordinatePhaseQueue < 1.0) {
                    // 如果以上不满足，则非协调相位二次排队需小于100%
                    evaluateList.add("该方案符合控制策略");
                } else {
                    evaluateList.add("该方案不符合控制策略");
                }
            }
        }
        evaluateList.add("协调方向平均速度" + coordSpeed + "km/h");
        evaluateList.add("协调方向平均延误" + cordDelay + "s");
        String s = evaluateList.get(0);
        if (s.contains("不符合")) {
            evaluateList.add("协调效果差");
        } else {
            evaluateList.add("协调效果良好");
        }
        res.setEvaluateList(evaluateList);

        // 生成曲线图
        List<String> timeArray = TimeArrayUtil.getCustomTimeIntervals(startHourMinuteStr, endHourMinuteStr, 5);
        if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.NO_PARK_PASS_RATE.getDescription())
                || Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.TRVAL_TIME.getDescription())
                || Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.CORD_RELIABILITY.getDescription())
                || Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getDescription())
                || Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.CORD_QUEUE_RATIO.getDescription())) {

            // 按时间粒度聚合绿波历史数据
            Map<String, List<GreenwaveHistPO>> groupedByTime = greenwaveHistPOList.stream()
                    .collect(Collectors.groupingBy(po -> {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTime(po.getGmtModified());
                        return getHourMinuteString(calendar);
                    }));
            String enumName = StrategyAndMetricsEnum.Metrics.getEnumNameByDescription(metricName);
            evaluateMetrics(enumName, timeArray, groupedByTime, res);

        } else {
            List<MainlineSchemeEvaluateVO.CurveElement> curveElementList = new ArrayList<>();
            // 按时间粒度聚合路口方向历史数据
            Map<String, List<CrossDirDataHistPO>> groupedByTime = totalCrossDirDataHistPOList.stream()
                    .collect(Collectors.groupingBy(po -> {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTimeInMillis((long) po.getBatchTime() * 1000);
                        return getHourMinuteString(calendar);
                    }));
            if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    OptionalDouble minAverage = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getGreenLightEfficiency)
                                    .average()
                                    .orElse(0.0))
                            .min();
                    int value = (int) (minAverage.orElse(0.0) * 100);

                    curveElement.setValue(value);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.STOP_TIMES.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfAverages = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getStopTimes)
                                    .average()
                                    .orElse(0.0))
                            .sum();

                    int value = (int) sumOfAverages;

                    curveElement.setValue(value);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfAverages = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getDelayTime)
                                    .average()
                                    .orElse(0.0))
                            .sum();

                    int value = (int) sumOfAverages;

                    curveElement.setValue(value);
                    curveElementList.add(curveElement);
                }
            } else if (Objects.equals(metricName, StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getDescription())) {
                for (String hourMinute : timeArray) {
                    MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
                    curveElement.setMetricTime(hourMinute);

                    List<CrossDirDataHistPO> crossDirDataHistPOList = groupedByTime.get(hourMinute);
                    double sumOfAverages = crossDirDataHistPOList.stream()
                            .collect(Collectors.groupingBy(CrossDirDataHistPO::getCrossId))
                            .entrySet().stream()
                            .mapToDouble(entry -> entry.getValue().stream()
                                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                                    .average()
                                    .orElse(0.0))
                            .average().orElse(0.0);

                    int value = (int) sumOfAverages;

                    curveElement.setValue(value);
                    curveElementList.add(curveElement);
                }
            }
            res.setCurveElementList(curveElementList);
        }
    }

    @NotNull
    private String getHourMinuteString(Calendar calendar) {
        int minute = calendar.get(Calendar.MINUTE);
        minute = (minute / 5) * 5;  // Round down to nearest 5 minutes
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return hourMinuteFormat.format(calendar.getTime());
    }

    private void evaluateMetrics(String metricName, List<String> timeArray,
                                 Map<String, List<GreenwaveHistPO>> groupedByTime, MainlineSchemeEvaluateVO res) {
        List<MainlineSchemeEvaluateVO.CurveElement> curveElementList = new ArrayList<>();
        for (String hourMinute : timeArray) {
            MainlineSchemeEvaluateVO.CurveElement curveElement = new MainlineSchemeEvaluateVO.CurveElement();
            curveElement.setMetricTime(hourMinute);
            List<GreenwaveHistPO> greenwaveHistPOListHourMinute = groupedByTime.get(hourMinute);

            ToDoubleFunction<GreenwaveHistPO> function;
            switch (metricName) {
                case "NO_PARK_PASS_RATE":
                    function = GreenwaveHistPO::getNoparkPassRate;
                    break;
                case "TRVAL_TIME":
                    function = GreenwaveHistPO::getTrvalTime;
                    break;
                case "CORD_RELIABILITY":
                    function = GreenwaveHistPO::getCordReliability;
                    break;
                case "UNCOORDINATE_PHASE_QUEUE":
                    function = GreenwaveHistPO::getUncoordinatePhaseQueue;
                    break;
                case "CORD_QUEUE_RATIO":
                    function = GreenwaveHistPO::getCordQueueRatio;
                    break;
                default:
                    throw new IllegalArgumentException("Unknown metric: " + metricName);
            }
            if (CollectionUtil.isNotEmpty(greenwaveHistPOListHourMinute)) {
                int value = calculateAverage(greenwaveHistPOListHourMinute, function);
                curveElement.setValue(value);
            } else {
                curveElement.setValue(0);
            }
            curveElementList.add(curveElement);
        }
        res.setCurveElementList(curveElementList);
    }

    private int calculateAverage(List<GreenwaveHistPO> list, ToDoubleFunction<GreenwaveHistPO> function) {
        return (int) (list.stream().mapToDouble(function).average().orElse(0.0) * 100);
    }

    private void buildTotalCrossDirHistList(
            Date boStartTime, Date boEndTime, String startHourMinuteStr, String endHourMinuteStr, String dirStr,
            Integer dir, List<CrossDirDataHistPO> totalCrossDirDataHistPOList, GreenwaveCrossPO greenwaveCrossPO) {

        String crossId = greenwaveCrossPO.getCrossId();
        Integer dirCode = dir;
        dirCode = convertDirCode(dirStr, crossId, dirCode);
        String boStartDayStr = dayFormat.format(boStartTime);
        String boEndDayStr = dayFormat.format(boEndTime);
        List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossDirAndTimeSection(
                crossId, dirCode, boStartDayStr, boEndDayStr, startHourMinuteStr, endHourMinuteStr);
        totalCrossDirDataHistPOList.addAll(crossDirDataHistPOList);
    }

    private Integer convertDirCode(String dirStr, String crossId, Integer dirCode) {
        if (Objects.equals(crossId, "13NF80B5QN0")) {
            // 霞景路方向转换
            if (Objects.equals(dirStr, "东")) {
                dirCode = 2; // 东北
            } else if (Objects.equals(dirStr, "西")) {
                dirCode = 6; // 西南
            }
        } else if (Objects.equals(crossId, "13NGH0B5RC0")) {
            // 洪山路方向转换
            if (Objects.equals(dirStr, "西")) {
                dirCode = 6; // 西南
            }
        }
        return dirCode;
    }


    private boolean isTimeInRange(Date date, String startStr, String endStr) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        Date startDate = sdf.parse(startStr);
        Date endDate = sdf.parse(endStr);

        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int hour = cal.get(Calendar.HOUR_OF_DAY);
        int minute = cal.get(Calendar.MINUTE);

        cal.setTime(startDate);
        int startHour = cal.get(Calendar.HOUR_OF_DAY);
        int startMinute = cal.get(Calendar.MINUTE);

        cal.setTime(endDate);
        int endHour = cal.get(Calendar.HOUR_OF_DAY);
        int endMinute = cal.get(Calendar.MINUTE);

        return (hour > startHour || (hour == startHour && minute >= startMinute))
                && (hour < endHour || (hour == endHour && minute <= endMinute));
    }

    private List<GreenwaveDetailVO.GreenwaveCross> buildGreenwaveCrossList(Integer greenwaveId) {
        List<GreenwaveDetailVO.GreenwaveCross> res = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrossList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        int size = greenwaveCrossList.size();
        for (int i = 0; i < size; i++) {
            GreenwaveCrossPO greenwaveCrossPO = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = new GreenwaveDetailVO.GreenwaveCross();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCross.setCrossId(crossId);

            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (baseCrossInfoPO == null) {
                throw new RuntimeException("无此路口基础信息");
            }
            greenwaveCross.setCrossName(baseCrossInfoPO.getName());
            greenwaveCross.setIsKeyRoute(greenwaveCrossPO.getIsKeyRoute());
            greenwaveCross.setSort(greenwaveCrossPO.getSort());

            // 到下一个路口距离
            Integer distanceToNextCross = null;
            if (i < size - 1) { // 不是最后一个路口
                double nextCrossLen = greenwaveCrossPO.getNextCrossLen();
                distanceToNextCross = (int) nextCrossLen;
            }
            greenwaveCross.setDistanceToNextCross(distanceToNextCross);

            res.add(greenwaveCross);
        }
        return res;
    }

    private List<MainlineSchemeAnalysisVO.GreenwaveData> buildGreenwaveData(List<GreenwaveHistPO> filteredList) {
        List<MainlineSchemeAnalysisVO.GreenwaveData> res = new ArrayList<>();
        List<GreenwaveHistPO> distinctList = filteredList.stream()
                .filter(distinctByKey(GreenwaveHistPO::getGreenwaveName))
                .collect(Collectors.toList());

        for (GreenwaveHistPO greenwaveHistPO : distinctList) {
            MainlineSchemeAnalysisVO.GreenwaveData greenwaveData = new MainlineSchemeAnalysisVO.GreenwaveData();
            greenwaveData.setName(greenwaveHistPO.getGreenwaveName());
            greenwaveData.setGreenwaveId(greenwaveHistPO.getId());

            Integer greenwaveId = greenwaveHistPO.getId();
            greenwaveData.setDirList(buildDirList(greenwaveId));

            res.add(greenwaveData);
        }

        return res;
    }

    private List<MainlineSchemeAnalysisVO.DirectionList> buildDirList(Integer greenwaveId) {
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        List<MainlineSchemeAnalysisVO.DirectionList> res = new ArrayList<>();

        String greenwaveName = greenwaveInfoPO.getName();
        String[] split = greenwaveName.split(" ");
        String dirName = split[1];
        Integer coordinateDir = greenwaveInfoPO.getDir();

        if (coordinateDir == 0) { // 正向
            MainlineSchemeAnalysisVO.DirectionList directionList = new MainlineSchemeAnalysisVO.DirectionList();
            directionList.setName(dirName + "（协调方向）");
            directionList.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList);
            MainlineSchemeAnalysisVO.DirectionList directionList1 = new MainlineSchemeAnalysisVO.DirectionList();
            String dirNameRev = reverseDirName(dirName, "向");
            directionList1.setName(dirNameRev + "（非协调方向）");
            directionList1.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList1);
        } else if (coordinateDir == 1) { // 反向
            MainlineSchemeAnalysisVO.DirectionList directionList = new MainlineSchemeAnalysisVO.DirectionList();
            directionList.setName(dirName + "（非协调方向）");
            directionList.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList);
            MainlineSchemeAnalysisVO.DirectionList directionList1 = new MainlineSchemeAnalysisVO.DirectionList();
            String dirNameRev = reverseDirName(dirName, "向");
            directionList1.setName(dirNameRev + "（协调方向）");
            directionList1.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList1);
        } else { // 双向
            MainlineSchemeAnalysisVO.DirectionList directionList = new MainlineSchemeAnalysisVO.DirectionList();
            directionList.setName(dirName + "（协调方向）");
            directionList.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList);
            MainlineSchemeAnalysisVO.DirectionList directionList1 = new MainlineSchemeAnalysisVO.DirectionList();
            String dirNameRev = reverseDirName(dirName, "向");
            directionList1.setName(dirNameRev + "（协调方向）");
            directionList1.setMetricList(buildMetricList(greenwaveId, coordinateDir));
            res.add(directionList1);
        }

        return res;
    }

    private List<MainlineSchemeAnalysisVO.Metric> buildMetricList(Integer greenwaveId, Integer coordinateDir) {
        List<MainlineSchemeAnalysisVO.Metric> res = new ArrayList<>();
        // 确定控制策略
        StrategyPO strategyPO = getStrategyPO(greenwaveId);
        String strategyCode = strategyPO.getStrategyCode();

        if (coordinateDir == 0 || coordinateDir == 1) { // 单向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                addMetrics(res);
                MainlineSchemeAnalysisVO.Metric metric = new MainlineSchemeAnalysisVO.Metric();
                metric.setMetricName(StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getDescription());
                metric.setIsShown(0);
                res.add(metric);
            } else {
                fillBalanceMetrics(res, strategyCode);
            }
        } else { // 双向绿波
            if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                addMetrics(res);
            } else fillBalanceMetrics(res, strategyCode);
        }

        return res;
    }

    private void fillBalanceMetrics(List<MainlineSchemeAnalysisVO.Metric> res, String strategyCode) {
        if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
            MainlineSchemeAnalysisVO.Metric metric = new MainlineSchemeAnalysisVO.Metric();
            metric.setMetricName(StrategyAndMetricsEnum.Metrics.CORD_QUEUE_RATIO.getDescription());
            metric.setIsShown(0);
            res.add(metric);

            MainlineSchemeAnalysisVO.Metric metric1 = new MainlineSchemeAnalysisVO.Metric();
            metric1.setMetricName(StrategyAndMetricsEnum.Metrics.UNCOORDINATE_PHASE_QUEUE.getDescription());
            metric1.setIsShown(0);
            res.add(metric1);
        }
    }

    private void addMetrics(List<MainlineSchemeAnalysisVO.Metric> res) {
        MainlineSchemeAnalysisVO.Metric metric = new MainlineSchemeAnalysisVO.Metric();
        metric.setMetricName(StrategyAndMetricsEnum.Metrics.NO_PARK_PASS_RATE.getDescription());
        metric.setIsShown(0);
        res.add(metric);

        MainlineSchemeAnalysisVO.Metric metric2 = new MainlineSchemeAnalysisVO.Metric();
        metric2.setMetricName(StrategyAndMetricsEnum.Metrics.TRVAL_TIME.getDescription());
        metric2.setIsShown(0);
        res.add(metric2);

        MainlineSchemeAnalysisVO.Metric metric3 = new MainlineSchemeAnalysisVO.Metric();
        metric3.setMetricName(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getDescription());
        metric3.setIsShown(1);
        res.add(metric3);

        MainlineSchemeAnalysisVO.Metric metric4 = new MainlineSchemeAnalysisVO.Metric();
        metric4.setMetricName(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getDescription());
        metric4.setIsShown(1);
        res.add(metric4);

        MainlineSchemeAnalysisVO.Metric metric5 = new MainlineSchemeAnalysisVO.Metric();
        metric5.setMetricName(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getDescription());
        metric5.setIsShown(1);
        res.add(metric5);

        MainlineSchemeAnalysisVO.Metric metric6 = new MainlineSchemeAnalysisVO.Metric();
        metric6.setMetricName(StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getDescription());
        metric6.setIsShown(1);
        res.add(metric6);

        MainlineSchemeAnalysisVO.Metric metric7 = new MainlineSchemeAnalysisVO.Metric();
        metric7.setMetricName(StrategyAndMetricsEnum.Metrics.CORD_RELIABILITY.getDescription());
        metric7.setIsShown(0);
        res.add(metric7);
    }

    private String reverseDirName(String input, String character) {
        int index = input.indexOf(character);
        if (index != -1) {
            String part1 = input.substring(index + character.length());  // Part after "向"
            String part2 = input.substring(0, index + character.length());  // Part before and including "向"
            return part1 + part2;
        }
        return input;
    }

    private <T> Predicate<T> distinctByKey(Function<? super T, ?> keyExtractor) {
        Set<Object> seen = ConcurrentHashMap.newKeySet();
        return t -> seen.add(keyExtractor.apply(t));
    }

    private List<String> calcEvaluateData(List<GreenwaveHistPO> filteredList) {
        int qualified = 0;
        int notQualified = 0;
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Double noparkPassRate = greenwaveHistPO.getNoparkPassRate();
            Double cordReliability = greenwaveHistPO.getCordReliability();
            Double uncoordinatePhaseQueue = greenwaveHistPO.getUncoordinatePhaseQueue();
            Double cordQueueRatio = greenwaveHistPO.getCordQueueRatio();
            Integer greenwaveId = greenwaveHistPO.getId();
            // 确定控制策略
            StrategyPO strategyPO = getStrategyPO(greenwaveId);
            String strategyCode = strategyPO.getStrategyCode();

            // 确定评价
            // 获取时段
            Integer sectionId = greenwaveHistPO.getSectionId();
            CrossSectionPO crossSectionPO = baseCrossSectionMapper.selectById(sectionId);
            String startHourMinuteStr = crossSectionPO.getStartTime();
            String endHourMinuteStr = crossSectionPO.getEndTime();
            Date day = greenwaveHistPO.getGmtModified();

            Date startDate = combineDateAndTime(day, startHourMinuteStr);
            Date endDate = combineDateAndTime(day, endHourMinuteStr);

            // 获取路口
            List<BaseCrossInfoPO> crossInfoPOList = getGreenwaveCross(greenwaveId);
            // 获取路口历史数据
            List<String> crossIdList = crossInfoPOList.stream()
                    .map(BaseCrossInfoPO::getId)
                    .collect(Collectors.toList());
            int startTimeStamp = (int) (startDate.getTime() / 1000);
            int endTimeStamp = (int) (endDate.getTime() / 1000);
            List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper
                    .selectByCrossIdsAndTimestamp(crossIdList, startTimeStamp, endTimeStamp);
            // 判断时段内的饱和度
            double saturation = crossDataHistPOList.stream()
                    .mapToDouble(CrossDataHistPO::getSturation)
                    .average()
                    .orElse(0.0);

            Integer dir = greenwaveHistPO.getDir();
            if (dir == 0 || dir == 1) { // 单向绿波
                if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                    if (saturation < 0.8) {
                        if (noparkPassRate > 0.8 && cordReliability > 0.7) {
                            // 协调不停车通过率需大于80%，并且协调方案可靠性需大于70%
                            qualified++;
                        } else if (uncoordinatePhaseQueue < 0.5) {
                            // 如果以上不满足，则非协调相位二次排队需小于50%
                            qualified++;
                        } else {
                            notQualified++;
                        }
                    } else {
                        // 协调方案可靠性需大于70%
                        if (cordReliability > 0.7) {
                            qualified++;
                        } else {
                            notQualified++;
                        }
                    }
                } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                    if (cordQueueRatio < 0.6) {
                        // 协调路段排队空间占比需小于60%
                        qualified++;
                    } else if (uncoordinatePhaseQueue < 1.0) {
                        // 如果以上不满足，则非协调相位二次排队需小于100%
                        qualified++;
                    } else {
                        notQualified++;
                    }
                }
            } else { // 双向绿波
                if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_EFFICIENCY.getCode())) {
                    // 协调不停车通过率需大于50%，并且协调方案可靠性需大于70%
                    if (noparkPassRate > 0.5 && cordReliability > 0.7) {
                        qualified++;
                    } else {
                        notQualified++;
                    }
                } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.LINE_BALANCE.getCode())) {
                    if (cordQueueRatio < 0.6) {
                        // 协调路段排队空间占比需小于60%
                        qualified++;
                    } else if (uncoordinatePhaseQueue < 1.0) {
                        // 如果以上不满足，则非协调相位二次排队需小于100%
                        qualified++;
                    } else {
                        notQualified++;
                    }
                }
            }
        }
        List<String> res = new ArrayList<>();
        res.add("符合控制策略方案数 " + qualified + "个");
        res.add("不符合控制策略方案数 " + notQualified + "个");
        return res;
    }

    private StrategyPO getStrategyPO(Integer greenwaveId) {
        GreenwaveScenePO greenwaveScenePO = greenwaveSceneMapper.selectByGreenwaveId(greenwaveId);
        Integer sceneId = greenwaveScenePO.getSceneId();
        List<SceneStrategyPO> sceneStrategyPOList = sceneStrategyMapper.selectBySceneId(sceneId);
        Optional<SceneStrategyPO> maxPriorityPO = sceneStrategyPOList.stream()
                .min(Comparator.comparingInt(SceneStrategyPO::getSceneStrategyPriority));
        SceneStrategyPO sceneStrategyPO = maxPriorityPO.get();
        Integer strategyId = sceneStrategyPO.getStrategyId();
        StrategyPO strategyPO = strategyMapper.selectById(strategyId);
        return strategyPO;
    }

    private List<BaseCrossInfoPO> getGreenwaveCross(Integer greenwaveId) {
        List<BaseCrossInfoPO> res = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrosses = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        for (GreenwaveCrossPO greenwaveCross : greenwaveCrosses) {
            String crossId = greenwaveCross.getCrossId();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            res.add(baseCrossInfoPO);
        }
        return res;
    }

    private Date combineDateAndTime(Date day, String hourMinuteStr) {
        String[] hourMinute = hourMinuteStr.split(":");
        int hour = Integer.parseInt(hourMinute[0]);
        int minute = Integer.parseInt(hourMinute[1]);

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(day);

        calendar.set(Calendar.HOUR_OF_DAY, hour);
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        return calendar.getTime();
    }

    private List<MainlineSchemeAnalysisVO.CrossData> calcCrossData(List<GreenwaveHistPO> filteredList) {
        List<MainlineSchemeAnalysisVO.CrossData> res = new ArrayList<>();

        // 找出绿波对应的关键路口
        Map<Integer, String> greenwaveCrossMap = new HashMap<>();
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Integer greenwaveId = greenwaveHistPO.getId();
            GreenwaveCrossPO greenwaveCrossPO = greenwaveCrossMapper.selectByGreenwaveIdAndKeyRoute(greenwaveId);
            Integer greenId = greenwaveCrossPO.getGreenId();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCrossMap.put(greenId, crossId);
        }

        // 统计关键路口次数
        Map<String, Integer> crossTimesMap = new HashMap<>();
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Integer greenwaveHistPOId = greenwaveHistPO.getId();
            String crossId = greenwaveCrossMap.get(greenwaveHistPOId);
            crossTimesMap.put(crossId, crossTimesMap.getOrDefault(crossId, 0) + 1);
        }

        for (Map.Entry<String, Integer> entry : crossTimesMap.entrySet()) {
            MainlineSchemeAnalysisVO.CrossData crossData = new MainlineSchemeAnalysisVO.CrossData();
            String crossId = entry.getKey();
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            String crossName = baseCrossInfoPO.getName();
            crossData.setName(crossName);
            crossData.setTimes(entry.getValue());
            res.add(crossData);
        }

        return res;
    }

    private List<MainlineSchemeAnalysisVO.DirectionData> calcSceneData(List<GreenwaveHistPO> filteredList) {
        List<MainlineSchemeAnalysisVO.DirectionData> res = new ArrayList<>();

        HashMap<String, Integer> map = new HashMap<>();
        for (GreenwaveHistPO greenwaveHistPO : filteredList) {
            Integer dir = greenwaveHistPO.getDir();
            if (dir == 0) { // 正向
                map.put("单向绿波上行", map.getOrDefault("单向绿波上行", 0) + 1);
            } else if (dir == 1) { // 反向
                map.put("单向绿波下行", map.getOrDefault("单向绿波下行", 0) + 1);
            } else { // 双向
                map.put("双向绿波", map.getOrDefault("双向绿波", 0) + 1);
            }
        }

        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            MainlineSchemeAnalysisVO.DirectionData directionData = new MainlineSchemeAnalysisVO.DirectionData();
            directionData.setName(entry.getKey());
            directionData.setTimes(entry.getValue());
            res.add(directionData);
        }

        return res;
    }

    private Date calcDate(String timeStr, Date startTime) throws ParseException {
        // 使用 Calendar 来获取年、月、日信息
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);
        // 使用 SimpleDateFormat 来获取时和分
        Date time = hourMinuteFormat.parse(timeStr);
        Calendar timeCal = Calendar.getInstance();
        timeCal.setTime(time);
        // 组合两者
        calendar.set(Calendar.HOUR_OF_DAY, timeCal.get(Calendar.HOUR_OF_DAY));
        calendar.set(Calendar.MINUTE, timeCal.get(Calendar.MINUTE));
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        // 转换回 Date 对象
        Date startDatetime = calendar.getTime();
        return startDatetime;
    }

    private List<String> buildTimeList(Date startTime, Date endTime, Integer minutes) {
        // 用于存储时间区间的列表
        List<String> timeList = new ArrayList<>();

        Calendar cal = Calendar.getInstance();
        cal.setTime(startTime);

        while (cal.getTime().before(endTime) || cal.getTime().equals(endTime)) {
            String formattedTime = hourMinuteFormat.format(cal.getTime());
            timeList.add(formattedTime);
            // 增加指定分钟数
            cal.add(Calendar.MINUTE, minutes);
        }
        return timeList;
    }

    public static String formatTime(int totalMinutes) {
        int hours = totalMinutes / 60;
        int minutes = totalMinutes % 60;
        return String.format("%02d:%02d", hours, minutes);
    }

    private List<String> buildLaneMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<LaneInfoPO> laneList = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dirType);
            for (LaneInfoPO lanePO : laneList) {
                Integer turn = lanePO.getTurn();
                String turnDesc = TurnConvertEnum.getDescByKey(turn);
                Integer sort = lanePO.getSort();
                res.add(dirName + "进口" + turnDesc + sort + "车道");
            }
        }
        return res;
    }

    private List<String> buildTurnMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<CrossTurnInfoPO> turnList = baseCrossTurnInfoMapper.selectByCrossIdAndDir(crossId, dirType);
            for (CrossTurnInfoPO turnPO : turnList) {
                String turnType = turnPO.getTurnType();
                String turnDesc = TurnConvertEnum.getDescByCode(turnType);
                res.add(dirName + "进口" + turnDesc);
            }
        }
        return res;
    }

    private List<String> buildDirMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> filteredList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO po : filteredList) {
            Integer dirType = po.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            res.add(dirName + "进口");
        }
        return res;
    }

}
